package CarTypeRecognition::Visualization;

use 5.008001;
use strict;
use warnings;

require Exporter;
use Image::Magick;
use Data::Dumper;
use File::Basename;
use Storable;
use Chart::Graph::Gnuplot qw(&gnuplot);
use Time::Piece::ISO;

use CarTypeRecognition::System;
use CarTypeRecognition::Misc;


our @ISA = qw(Exporter);
our $VERSION = '0.01';


sub new($$)
{
	my $proto = shift;
	my ( $props ) = @_;

	my $class = ref($proto) || $proto;
	my $s =CarTypeRecognition::System->new({'-base-dir'=>$props->{'-base-dir'}}) || die "Cannot create System object\n";
	my $im=CarTypeRecognition::Image->new ||  die "Cannot create Image object!\n";
	my $m =CarTypeRecognition::Misc->new({'-dir'=>$props->{'-base-dir'}||'./'});

	
	my $self  = {
		'_SYSTEM'=>$s,
		'_IM_OBJ'=>$im,
		'_MISC_OBJ'=>$m,
		'_VIZ_DIR'=>$props->{'-dir'} || 'report',
		'_SIFT'=>$props->{'-sift'},
	};

	`mkdir -p $self->{'_VIZ_DIR'}`;

	bless ($self, $class);
	return $self;
}

sub get_base_dir($)
{
	return shift->{'-base-dir'};
}

sub get_miscobj($)
{
	return shift->{'_MISC_OBJ'};
}

sub get_system($)
{
	my $self = shift;
	return $self->{'_SYSTEM'};
}

sub get_imqobj($)
{
	my $self = shift;
	return $self->{'_IM_OBJ'};
}
sub get_sift($)
{
	my $self = shift;
	return $self->{'_SIFT'};
}

sub _get_save_filename($$$)
{
	my $self=shift;
	my ($car_filename, $ext)=@_;
	my $sift=$self->get_sift;
	my $f=basename $car_filename;
	return $self->{'_VIZ_DIR'} . "/$f-$ext" .$sift->get_config() . ".png";
}

sub get_save_filename_sift($$)
{
	my $self=shift;
	my ($car_filename)=@_;
	return $self->_get_save_filename( $car_filename, 'sift_viz' );
}

sub get_save_filename_variance_wclass($$)
{
	my $self=shift;
	my ($car_filename)=@_;
	return $self->_get_save_filename( $car_filename, 'variance_wclass' );
}

sub drawbox_sifthist($$$$)
{
	my $self=shift;
	my ($p,$feature,$coords)=@_;

#	print Dumper $coords;
	my $x=$coords->[0];
	my $y=$coords->[1];
	my $w=$coords->[2];
	my $h=$coords->[3];


	my $xe=$x+$w;
	my $ye=$y+$h;

	my $feat_num=@$feature;
	my $bw=$w/$feat_num;

	# number of bins > $w pixels 
	warn "number of bins do not fit into the box!\n" if $feat_num>$w;

	$p->Draw(primitive=>'Rectangle',fill=>'none',stroke=>'black',
			strokewidth=>1,points=>"$x $y $xe $ye");

	my $xoff=$x;
	for my $v ( @$feature ) {
		my $x1=$xoff;
		my $y2=$y+$h-1;
		my $x2=$x1+$bw;
		my $y1=$y+$h-$v*$h-1;
		$p->Draw(primitive=>'Rectangle',fill=>'gray',stroke=>'black',
				strokewidth=>1,points=>"$x1 $y1 $x2 $y2");
		$xoff+=$bw;				
	}

}

# This does the classification process 
# i.e.: creates sifts for each car
sub visualize_SIFT($$)
{
	my $self=shift;
	my ($car_filename)=@_;
	my $s   =$self->get_system();
	my $sift=$self->get_sift();
	my $im  =$self->get_imqobj();
				
	my $bv=$sift->{'_BOXES_VERT'};
	my $bh=$sift->{'_BOXES_HORIZ'};
	my $f=$self->get_save_filename_sift($car_filename);	
	print "\nSaving into $f ...\n";
	return $f if -f $f; # Do not re-generate existing files
	
	print "Loading/Creating SIFT ... \n";	
	my $sift_features=$sift->create_SIFT($car_filename);

	print "Loading SIFT coords ... \n";	
	my $sift_boxes   =$sift->get_coords($car_filename);
			
	print "Loading Image ($car_filename ) ... \n";
	my ($p, $ow, $oh) = $im->load_image( $car_filename );

	my $gb=$sift->{'_GRAD_BINS'};
	$gb=1 if $gb<1;
	my $c=0;

	my $n1=@$sift_boxes;
	my $n2=@$sift_features;

	die "visualize_SIFT(): Number of sift boxes($n1) differs from the number of sift features($n2)!\n" unless $n1==$n2;

#$p->Draw(primitive=>'Rectangle',fill=>'white',stroke=>'white',
#			strokewidth=>1,points=>"0 0 256 92");
	for my $feature ( @$sift_features ) {
		my $coords = shift @$sift_boxes;
		$self->drawbox_sifthist( $p,$feature, $coords );
	}
	$p->Write($f);

	return $f;
}

# Variance with the class visualization
sub visualize_variance_within_class($$)
{
	my $self=shift;
	my ($class_name)=@_;
	my $s   =$self->get_system();
	my $sift=$self->get_sift();
	my $im  =$self->get_imqobj();
	my $misc=$self->get_miscobj();
	my $db_dir=$self->get_base_dir();
				
	my $bv=$sift->{'_BOXES_VERT'};
	my $bh=$sift->{'_BOXES_HORIZ'};

	my @cars_from_class=$s->get_cars_for_class($class_name, $db_dir);
	my (undef, $p)=$misc->average_car($class_name);

	my $f=$self->get_save_filename_variance_wclass($class_name);	
	my @data_to_process=();
	for(my $i=0; $i<$bv*$bh; $i++ ) {
		for my $car_filename ( @cars_from_class ) {
			print "Processing $car_filename ...\n";
			return $f if -f $f; # Do not re-generate existing files
	
			print "Loading/Creating SIFT ... \n";	
			my $sift_features=$sift->create_SIFT($car_filename);
			
			$data_to_process[$i]=[] unless $data_to_process[$i];
			push( @{$data_to_process[$i]}, $sift_features->[$i] );
		}
	}

#	print "data_to_process: " . Dumper \@data_to_process;
	my $size=$s->abs_v(\@{$s->variance(@data_to_process)});
	print "Loading SIFT coords ... \n";	

#	my $sift_boxes   =$sift->get_coords($car_filename);
#			
#			my $gb=$sift->{'_GRAD_BINS'};
#			$gb=1 if $gb<1;
#			my $c=0;
#
#			my $n1=@$sift_boxes;
#			my $n2=@$sift_features;
#
#			die "visualize_SIFT(): Number of sift boxes($n1) differs from the number of sift features($n2)!\n" unless $n1==$n2;
#
#			for my $feature ( @$sift_features ) {
#				my $coords = shift @$sift_boxes;
#				$self->drawbox_sifthist( $p,$feature, $coords );
#			}
#	print "\nSaving into $f ...\n";
#	$p->Write($f);
#
	return $f;
}





# Preloaded methods go here.

1;
__END__
# Below is stub documentation for your module. You'd better edit it!

=head1 NAME

CarTypeRecognition::Main - Perl extension for blah blah blah

=head1 SYNOPSIS

  use CarTypeRecognition::Main;
  blah blah blah

=head1 DESCRIPTION

Stub documentation for CarTypeRecognition::Main, created by h2xs. It looks like the
author of the extension was negligent enough to leave the stub
unedited.

Blah blah blah.

=head2 EXPORT

None by default.



=head1 SEE ALSO

Mention other useful documentation such as the documentation of
related modules or operating system documentation (such as man pages
in UNIX), or any relevant external documentation such as RFCs or
standards.

If you have a mailing list set up for your module, mention it here.

If you have a web site set up for your module, mention it here.

=head1 AUTHOR

root, E<lt>root@localdomainE<gt>

=head1 COPYRIGHT AND LICENSE

Copyright (C) 2005 by root

This library is free software; you can redistribute it and/or modify
it under the same terms as Perl itself, either Perl version 5.8.1 or,
at your option, any later version of Perl 5 you may have available.


=cut
