#!/usr/bin/perl

use warnings;
use strict;
use Data::Dumper;

use CarTypeRecognition::Classification::kNN;
use CarTypeRecognition::Classification::LDA;
use CarTypeRecognition::SIFT;
use CarTypeRecognition::SIFT::NormalizeGrad;
use CarTypeRecognition::SIFT::NormalizeGradBlur;
use CarTypeRecognition::Report;
use CarTypeRecognition::Visualization;
require CarTypeRecognition::Error::Euclidean;	
require CarTypeRecognition::Error::EMD;	
use Getopt::Long;

my $hbn;
my $vbn;
my $bins;
my $opt_euclidean_dist=1;
my $opt_emd_distance=0;
my $opt_knn=1;
my $knn_k=1;
my $opt_sift_overlap=1;
my $opt_help=0;
my $sift_type='normalized-grad-blur';
my $db_dir;
my $opt_lda=0;

my $help=<<EOH;
$0 [options] <directory>
	--horizontal-box-num|h  : Number of horizontal boxes ( > 2 )
	--vertical-box-num|v    : Number of vertical boxes ( > 2 )
	--bins|b                : Number of bins in one box ( > 2 )
	--emd!                  : Use EMD as an error function
	--euclidean!            : Use Euclidean distance as an error function (default)
	--knn!                  : Use k-NN as a classifier (default)
	--lda!                  : Use LDA as a classifier
	--knn-k|k               : Choose paramaeter k ( >0 )
	--sift-overlap!         : Select SIFT overlapping (default)
	--sift-type|t           : one of simple, normalized-grad and normalized-grad-blur(default)
	--db-dir|d              : Car database directory
	--help                  : This help
EOH

my %Options=(
			'--horizontal-box-num|h=s'   => \$hbn,
			'--vertical-box-num|v=s'   => \$vbn,
			'--bins|b=s'   => \$bins,
			'--emd!' => \$opt_emd_distance,
			'--euclidean!' => \$opt_euclidean_dist,
			'--knn!' => \$opt_knn,
			'--lda!' => \$opt_lda,
			'--knn-k|k=i' => \$knn_k,
			'--sift-overlap!' => \$opt_sift_overlap,
			'--db-dir|d=s' => \$db_dir,
			'--help' => \$opt_help,
			'--sift-type|t=s' => \$sift_type,
			);

sub validate_config()
{
	$opt_euclidean_dist=0 if( $opt_emd_distance );
	die "You can select either Euclidean or EMD distance error function!\n" if( $opt_euclidean_dist && $opt_emd_distance );
	die "Number of bins must be at least 2!\n" if $bins<2;
	die "Number of horizontal boxes must be at least 2\n" if $hbn<2;
	die "Number of vertical boxes must be at least 2\n" if $vbn<2;
	die "Number k in k-NN classfier must be >=1\n" if $knn_k<1;
	die "You must supply cars db directory!\n" unless $db_dir;
	die "Cars db dir not exists!\n" unless -d $db_dir;
	die "SIFT type is one of (simple, normalized-grad, normalized-grad-blur)\n" 
		unless $sift_type eq 'simple' || $sift_type eq 'normalized-grad' || $sift_type eq 'normalized-grad-blur';
}

sub print_config()
{
	my $c_method='k-NN' if $opt_knn;
	$c_method='LDA' if $opt_lda;
	my $c_k=$knn_k;
	my $err_f="Euclidean" if $opt_euclidean_dist;
	$err_f="EMD" if $opt_emd_distance;

	print "Classification method: $c_method\n";
	print "k-NN parameter: $knn_k\n" if $opt_knn;
	print "SIFT is: ${hbn}x${vbn}x$bins\n";
	print "SIFT type: $sift_type\n";
	print "SIFT overlap selected\n" if $opt_sift_overlap;
	print "Car database: $db_dir\n";
}

die $help if @ARGV==0;
GetOptions( %Options ) or die "GetOptions failure: $!"; 
die $help if $opt_help;
validate_config();
print_config();

my $overlap="overlap";
$overlap="no_overlap" unless $opt_sift_overlap;

my @combinations=( 
		{'gb'=>$bins, 'bxh'=>$hbn, 'bxv'=>$vbn},
		);


my @graphs=();
my $o=undef;

my $err;
my $sift;
my $errfunc_name;
my $cls;
my $one_to_one;
my $success;
for my $h (@combinations){
	#my $sift=CarTypeRecognition::SIFT->new({
	#	'-grad-bins'  =>  $h->{'gb'},
	#	'-boxes-horiz'=>  $h->{'bxh'},
	#	'-boxes-vert' =>  $h->{'bxv'},
	#	'-sift-dir'   => "$ENV{HOME}/.car-recognition/sift",
	#	});
    #
	
	if( $opt_emd_distance ) {
		$errfunc_name="EMD";
		$err =CarTypeRecognition::Error::EMD->new({
			'-emd-prog'=>'emd/compute-SIFT-emd',
			});	
	}
	elsif($opt_euclidean_dist) {
		$errfunc_name="Euclidean";
		$err =CarTypeRecognition::Error::Euclidean->new();
	}
	else {
		die "Unknwon error func!\n";
	}


	if( $sift_type eq 'normalized-grad-blur' ) {
		$sift=CarTypeRecognition::SIFT::NormalizeGradBlur->new({
			'-grad-bins'  =>  $h->{'gb'},
			'-boxes-horiz'=>  $h->{'bxh'},
			'-boxes-vert' =>  $h->{'bxv'},
			'-sift-dir'   => "$ENV{HOME}/.car-recognition/sift",
			'-error-func' => $err,
			'-overlap'    => $opt_sift_overlap,
			});
	}
	elsif( $sift_type eq 'normalized-grad' ) {
		$sift=CarTypeRecognition::SIFT::NormalizeGrad->new({
			'-grad-bins'  =>  $h->{'gb'},
			'-boxes-horiz'=>  $h->{'bxh'},
			'-boxes-vert' =>  $h->{'bxv'},
			'-sift-dir'   => "$ENV{HOME}/.car-recognition/sift",
			'-error-func' => $err,
			'-overlap'    => $opt_sift_overlap,
			});
	}
	elsif( $sift_type eq 'simple' ) {
		$sift=CarTypeRecognition::SIFT->new({
			'-grad-bins'  =>  $h->{'gb'},
			'-boxes-horiz'=>  $h->{'bxh'},
			'-boxes-vert' =>  $h->{'bxv'},
			'-sift-dir'   => "$ENV{HOME}/.car-recognition/sift",
			'-error-func' => $err,
			'-overlap'    => $opt_sift_overlap,
			});
	}
	else {
		die "Unknwon SIFT type!\n";
	}
    
	my $v=CarTypeRecognition::Visualization->new( { '-sift' => $sift, '-dir'=> 'report' } );
	my $c; # classificator
	if( $opt_lda ){
		print "LDA Classifier used\n";
		$c=CarTypeRecognition::Classification::LDA->new({
				'-sift'         => $sift,
				'-db-directory' => $db_dir,
				});
	}
	elsif( $opt_knn ) {
		print "k-NN Classifier used\n";
		$c=CarTypeRecognition::Classification::kNN->new({
				'-sift'         => $sift,
				'-db-directory' => $db_dir,
				'-k'            => $knn_k,
				});
	}
	else {
		die "Unknown classification method!\n";
	}	
	($cls, $one_to_one, $success) = $c->classification_process(\@graphs);
	print "SUCCESS:".Dumper $success;
	print "Saving results...\n";
# Save result
	local *F;
	open F, "+>> experiments";
	print F	"\n$db_dir|$sift_type-${hbn}x${vbn}x$bins-$knn_k-NN-$overlap-$errfunc_name|$success->{'1'}";
	close F;
	my $r=CarTypeRecognition::Report->new({
								'-viz-obj' => $v,	
								'-dir' => './',
								'-report-me'=>$one_to_one,
								'-sift'=>$sift,
								'-name'=>$sift->get_config."-$errfunc_name",
								});
$r->report_cross($knn_k);
}
my $rep_graph=CarTypeRecognition::Report->new({
		'-name'=>"$sift_type-${hbn}x${vbn}x$bins-$knn_k-NN-$overlap-$errfunc_name",
		});
$rep_graph->report_gnuplot( \@graphs );


