
/*
 * Guillaume Cottenceau (gc at mandrakesoft.com)
 *
 * Copyright 2002 MandrakeSoft
 *
 * This software may be freely redistributed under the terms of the GNU
 * public license.
 *
 */

#include <unistd.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <stdarg.h>
#include <math.h>
#include <unistd.h>


#include "image.h"
#include "classification.h"

void abort_(const char * s, ...)
{
	va_list args;
	va_start(args, s);
	vfprintf(stderr, s, args);
	fprintf(stderr, "\n");
	va_end(args);
	abort();
}


static image_t pictures[MAX_PICTURES];

static void read_png_file(char* file_name, int pic_num);

void
read_images(char **filenames, int n)
{
	int pic_num;
	for(pic_num=0;pic_num<n;pic_num++){
		fprintf(stderr, "Reading images: %.2f%% done.\r", (double)100*pic_num/n); fflush(stderr);
		read_png_file(filenames[pic_num], pic_num);
		set_img_type(filenames[pic_num], pic_num);
	}
	fprintf(stderr, "\n");
}

static void read_png_file(char* file_name, int pic_num)
{
	char header[8];	// 8 is the maximum size that can be checked
	png_bytep * row_pointers;
	int y;
	png_structp png_ptr;
	png_infop info_ptr;
	int number_of_passes;
	int width, height;

	/* open file and test for it being a png */
	FILE *fp = fopen(file_name, "rb");
	if (!fp)
		abort_("[read_png_file] File %s could not be opened for reading", file_name);
	fread(header, 1, 8, fp);
	if (png_sig_cmp(header, 0, 8))
		abort_("[read_png_file] File %s is not recognized as a PNG file", file_name);


	/* initialize stuff */
	png_ptr = png_create_read_struct(PNG_LIBPNG_VER_STRING, NULL, NULL, NULL);
	
	if (!png_ptr)
		abort_("[read_png_file] png_create_read_struct failed");

	info_ptr = png_create_info_struct(png_ptr);
	if (!info_ptr)
		abort_("[read_png_file] png_create_info_struct failed");

	if (setjmp(png_jmpbuf(png_ptr)))
		abort_("[read_png_file] Error during init_io");

	png_init_io(png_ptr, fp);
	png_set_sig_bytes(png_ptr, 8);

	png_read_info(png_ptr, info_ptr);

	width=pictures[pic_num].w = info_ptr->width;
	height=pictures[pic_num].h = info_ptr->height;
	pictures[pic_num].color_type = info_ptr->color_type;
	pictures[pic_num].bit_depth = info_ptr->bit_depth;

	switch(info_ptr->color_type){
		case PNG_COLOR_TYPE_RGBA:
			pictures[pic_num].pixel_width=4;
			break;
		case PNG_COLOR_TYPE_RGB:
			pictures[pic_num].pixel_width=3;
			break;
		default:	
			abort_("ERROR:[process_file] color_type of input file must be PNG_COLOR_TYPE_RGBA (is %d)", info_ptr->color_type);
	}

	number_of_passes = png_set_interlace_handling(png_ptr);
	png_read_update_info(png_ptr, info_ptr);


	/* read file */
	if (setjmp(png_jmpbuf(png_ptr)))
		abort_("[read_png_file] Error during read_image");

	row_pointers = (png_bytep*) malloc(sizeof(png_bytep) * height);
	for (y=0; y<height; y++)
		row_pointers[y] = (png_byte*) malloc(info_ptr->rowbytes);

	png_read_image(png_ptr, row_pointers);

	pictures[pic_num].row_pointers=row_pointers;

        fclose(fp);
}


#if 0
void write_png_file(char* file_name)
{
	/* create file */
	FILE *fp = fopen(file_name, "wb");
	int y;
	if (!fp)
		abort_("[write_png_file] File %s could not be opened for writing", file_name);


	/* initialize stuff */
	png_ptr = png_create_write_struct(PNG_LIBPNG_VER_STRING, NULL, NULL, NULL);
	
	if (!png_ptr)
		abort_("[write_png_file] png_create_write_struct failed");

	info_ptr = png_create_info_struct(png_ptr);
	if (!info_ptr)
		abort_("[write_png_file] png_create_info_struct failed");

	if (setjmp(png_jmpbuf(png_ptr)))
		abort_("[write_png_file] Error during init_io");

	png_init_io(png_ptr, fp);


	/* write header */
	if (setjmp(png_jmpbuf(png_ptr)))
		abort_("[write_png_file] Error during writing header");

	png_set_IHDR(png_ptr, info_ptr, width, height,
		     bit_depth, color_type, PNG_INTERLACE_NONE,
		     PNG_COMPRESSION_TYPE_BASE, PNG_FILTER_TYPE_BASE);

	png_write_info(png_ptr, info_ptr);


	/* write bytes */
	if (setjmp(png_jmpbuf(png_ptr)))
		abort_("[write_png_file] Error during writing bytes");

	png_write_image(png_ptr, row_pointers);


	/* end write */
	if (setjmp(png_jmpbuf(png_ptr)))
		abort_("[write_png_file] Error during end of write");

	png_write_end(png_ptr, NULL);

        /* cleanup heap allocation */
	for (y=0; y<height; y++)
		free(row_pointers[y]);
	free(row_pointers);

        fclose(fp);
}
#endif

static __inline__ rgba_t * 
get_RGBA(int x, int y, int pic_num){
	int pixel_width=pictures[pic_num].pixel_width;
	png_byte* row = pictures[pic_num].row_pointers[y];
	png_byte* ptr = &(row[x*pixel_width]);
	rgba_t *rgba=malloc(sizeof(*rgba));	
	rgba->r=ptr[0];
	rgba->g=ptr[1];
	rgba->b=ptr[2];
	if(pixel_width==4){
		rgba->a=ptr[3];
	}
	return rgba;
}

static __inline__ void 
set_RGBA(int x, int y, rgba_t *rgba, int pic_num){
	int pixel_width=pictures[pic_num].pixel_width;
	png_byte* row = pictures[pic_num].row_pointers[y];
	png_byte* ptr = &(row[x*pixel_width]);
	ptr[0]=rgba->r;
	ptr[1]=rgba->g;
	ptr[2]=rgba->b;
	if( pixel_width == 4 )
	ptr[3]=rgba->a;
}

__inline__ double Iv(double x, double y, int pic_num)
{
	int xx=(int)floor(x+0.5);
	int yy=(int)floor(y+0.5);
	rgba_t *rgba=get_RGBA(xx,yy,pic_num);
	return (0.3*rgba->r + 0.6*rgba->g + 0.1*rgba->b);
}

__inline__ int get_image_width(int pic_num)
{
	return pictures[pic_num].w;
}


__inline__ int get_image_height(int pic_num)
{
	return pictures[pic_num].h;
}
