
/*
 * Guillaume Cottenceau (gc at mandrakesoft.com)
 *
 * Copyright 2002 MandrakeSoft
 *
 * This software may be freely redistributed under the terms of the GNU
 * public license.
 *
 */

#include <unistd.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <stdarg.h>
#include <math.h>
#include <unistd.h>

#define PNG_DEBUG 3
#include <png.h>

static char *image_to_process;
static char *dir_to_process;
static char *sift_type_str;
static int sift_overlap=0;

float var=0.01;

typedef struct {
	int         w, h;
	png_byte    color_type;
	png_bytep * row_pointers;
	png_byte    bit_depth;
	int         pixel_width;
} image_t;

void abort_(const char * s, ...)
{
	va_list args;
	va_start(args, s);
	vfprintf(stderr, s, args);
	fprintf(stderr, "\n");
	va_end(args);
	abort();
}


static image_t picture;

static void read_png_file(char* file_name)
{
	char header[8];	// 8 is the maximum size that can be checked
	png_bytep * row_pointers;
	int y;
	png_structp png_ptr;
	png_infop info_ptr;
	int number_of_passes;
	int width, height;

	/* open file and test for it being a png */
	FILE *fp = fopen(file_name, "rb");
	if (!fp)
		abort_("[read_png_file] File %s could not be opened for reading", file_name);
	fread(header, 1, 8, fp);
	if (png_sig_cmp(header, 0, 8))
		abort_("[read_png_file] File %s is not recognized as a PNG file", file_name);


	/* initialize stuff */
	png_ptr = png_create_read_struct(PNG_LIBPNG_VER_STRING, NULL, NULL, NULL);
	
	if (!png_ptr)
		abort_("[read_png_file] png_create_read_struct failed");

	info_ptr = png_create_info_struct(png_ptr);
	if (!info_ptr)
		abort_("[read_png_file] png_create_info_struct failed");

	if (setjmp(png_jmpbuf(png_ptr)))
		abort_("[read_png_file] Error during init_io");

	png_init_io(png_ptr, fp);
	png_set_sig_bytes(png_ptr, 8);

	png_read_info(png_ptr, info_ptr);

	width=picture.w = info_ptr->width;
	height=picture.h = info_ptr->height;
	picture.color_type = info_ptr->color_type;
	picture.bit_depth = info_ptr->bit_depth;

	switch(info_ptr->color_type){
		case PNG_COLOR_TYPE_GRAY:
			picture.pixel_width=1;
			break;
		case PNG_COLOR_TYPE_RGBA:
			picture.pixel_width=4;
			break;
		case PNG_COLOR_TYPE_RGB:
			picture.pixel_width=3;
			break;
		default:	
			abort_("ERROR:[process_file] color_type of input file must be PNG_COLOR_TYPE_RGBA (is %d)", info_ptr->color_type);
	}

	number_of_passes = png_set_interlace_handling(png_ptr);
	png_read_update_info(png_ptr, info_ptr);


	/* read file */
	if (setjmp(png_jmpbuf(png_ptr)))
		abort_("[read_png_file] Error during read_image");

	row_pointers = (png_bytep*) malloc(sizeof(png_bytep) * height);
	for (y=0; y<height; y++)
		row_pointers[y] = (png_byte*) malloc(info_ptr->rowbytes);

	png_read_image(png_ptr, row_pointers);

	picture.row_pointers=row_pointers;

        fclose(fp);
}


void write_png_file(char* file_name)
{
	png_bytep * row_pointers=picture.row_pointers;
	png_structp png_ptr;
	png_infop info_ptr;
	int width=picture.w;
	int height=picture.h;
	png_byte color_type=picture.color_type;
	png_byte bit_depth=picture.bit_depth;
	/* create file */
	FILE *fp = fopen(file_name, "wb");
	int y;
	if (!fp)
		abort_("[write_png_file] File %s could not be opened for writing", file_name);


	/* initialize stuff */
	png_ptr = png_create_write_struct(PNG_LIBPNG_VER_STRING, NULL, NULL, NULL);
	
	if (!png_ptr)
		abort_("[write_png_file] png_create_write_struct failed");

	info_ptr = png_create_info_struct(png_ptr);
	if (!info_ptr)
		abort_("[write_png_file] png_create_info_struct failed");

	if (setjmp(png_jmpbuf(png_ptr)))
		abort_("[write_png_file] Error during init_io");

	png_init_io(png_ptr, fp);


	/* write header */
	if (setjmp(png_jmpbuf(png_ptr)))
		abort_("[write_png_file] Error during writing header");

	png_set_IHDR(png_ptr, info_ptr, width, height,
		     bit_depth, color_type, PNG_INTERLACE_NONE,
		     PNG_COMPRESSION_TYPE_BASE, PNG_FILTER_TYPE_BASE);

	png_write_info(png_ptr, info_ptr);


	/* write bytes */
	if (setjmp(png_jmpbuf(png_ptr)))
		abort_("[write_png_file] Error during writing bytes");

	png_write_image(png_ptr, row_pointers);


	/* end write */
	if (setjmp(png_jmpbuf(png_ptr)))
		abort_("[write_png_file] Error during end of write");

	png_write_end(png_ptr, NULL);

        fclose(fp);
}
typedef struct{
	int r,g,b,a;
} rgba_t;

static __inline__ rgba_t * 
get_RGBA(int x, int y){
	int pixel_width=picture.pixel_width;
	png_byte* row = picture.row_pointers[y];
	png_byte* ptr = &(row[x*pixel_width]);
	rgba_t *rgba=malloc(sizeof(*rgba));	
	rgba->r=rgba->g=rgba->b=ptr[0]; /* Grayscale */
	if(pixel_width>1){              /* RGB */
		rgba->g=ptr[1];
		rgba->b=ptr[2];
	}
	if(pixel_width==4){             /* RGBA */
		rgba->a=ptr[3];
	}
	return rgba;
}

static __inline__ void 
set_RGBA(int x, int y, rgba_t *rgba){
	int pixel_width=picture.pixel_width;
	png_byte* row = picture.row_pointers[y];
	png_byte* ptr = &(row[x*pixel_width]);
	ptr[0]=rgba->r;
	if(pixel_width>1){
		ptr[1]=rgba->g;
		ptr[2]=rgba->b;
	}
	if( pixel_width == 4 )
		ptr[3]=rgba->a;
}

static void 
process_picture(void)
{
int bin_num=0;
double stw, sth;
double x,y;
int width=picture.w;
int height=picture.h;

srand(time(NULL));
	
	for (x=0; x<width; x++) {
		for (y=0; y<height; y++) {
			double n=var*rand()/RAND_MAX;
			rgba_t *rgba=get_RGBA(x,y);
			rgba->r+=(int)256*n;
			rgba->g+=(int)256*n;
			rgba->b+=(int)256*n;
			if(rgba->r>255) rgba->r=255;
			if(rgba->g>255) rgba->g=255;
			if(rgba->b>255) rgba->b=255;
			set_RGBA(x,y,rgba);
			free(rgba);
		}
	}
}



int main(int argc, char **argv)
{
	var=atof(argv[3]);
	printf("var=%.3f\n",var);
	read_png_file(argv[1]);
	process_picture();
	write_png_file(argv[2]);

	return 0;
}


